@@ -20,6 +20,12 @@ dtireg.smooth <- function(object,hmax=5,hinit=1,lambda=30,rho=1,graph=FALSE,slic
   }
   args <- sys.call(-3)
   args <- c(object@call,args)
+  sdcoef <- object@sdcoef
+  if(all(sdcoef[1:4]==0)) {
+    cat("No parameters for model of error standard deviation found\n estimating these parameters\n You may prefer to run sdpar before calling dtiTensor")
+    sdcoef <- sdpar(object,interactive=FALSE)@sdcoef
+    object@sdcoef <- sdcoef
+  }
   dtobject <- dtiTensor(object,method="nonlinear",varmethod=varmethod,varmodel=varmodel)
   scale <- dtobject@scale
   mask <- dtobject@mask
@@ -34,7 +40,6 @@ dtireg.smooth <- function(object,hmax=5,hinit=1,lambda=30,rho=1,graph=FALSE,slic
   ngrad <- object@ngrad
   ddim0 <- object@ddim0
   ddim <- object@ddim
-  sdcoef <- object@sdcoef
   z <- .Fortran("outlier",
                 as.double(object@si),
                 as.integer(prod(ddim)),
