@@ -5,6 +5,9 @@
 #include <R_ext/Utils.h>
 #include <stdlib.h>
 
+#include <sys/time.h>
+#include <omp.h>
+#include <time.h>
 
 int dim_x = 0, dim_y = 0, dim_z = 0, ngrad0c = 0;
 int i1 = 0, i2 = 0, i3 = 0;
@@ -148,8 +151,8 @@ int contains_int(int* array, int length, int value){
   return 0;
 }
 
-
-double* transpose(double* matrix, int dim1, int dim2){
+// wird nicht aufgerufen
+/* double* transpose(double* matrix, int dim1, int dim2){
   int i, j;
   double* t_matrix = Calloc(dim1*dim2, double);
   for(i = 0; i < dim1; i++){
@@ -157,8 +160,9 @@ double* transpose(double* matrix, int dim1, int dim2){
       t_matrix[get_ind2d(i,j,dim1)] = matrix[get_ind2d(j,i,dim2)];
     }
   }
+
   return t_matrix;
-}
+} */
   
 
 void paroforient(double *dir, double *angles){
@@ -207,6 +211,7 @@ double mfunpl0(int param_length, double *param, void* ex){
   Free(z);
   Free(w);
   Free(siq);
+
   return result;
 }
 
@@ -332,8 +337,8 @@ mfunplwghts0_ret mfunplwghts0(int param_length, double* param, double* siq){
 		    &pen, tmp, w, &tmp2);
 			
   tmp2 = 0;
-  Free(tmp);
-  tmp = Calloc(ngrad0c*m, double);
+//  Free(tmp);
+//  tmp = Calloc(ngrad0c*m, double);
 
   for(i = 0; i < m; i++){
     if(w[i] > 0){
@@ -565,7 +570,16 @@ mfunplwghts0_ret mfunplwghts0h(int param_length, double* param, double* siq){
   ret_val.value = result;
   ret_val.orient = or;  
   ret_val.mix = mix;
-  
+// begin einfuegung 
+Free(param_work);
+Free(z);
+Free(w);
+Free(w_red);
+Free(w_red2);
+Free(b);
+Free(work1);
+Free(o);
+// end einfuegung 
   return ret_val; 
 }
 
@@ -768,6 +782,9 @@ mfunplwghts0_ret mfunwghtsi(int param_length, double* param, double siq){
 		param[2*i+2] = or[2*i+1];
 	}
 	
+// begin einfuegung 
+//        Free(w2)
+// end einfuegung 
 	w2 = (double*) R_alloc(c_ord+1, sizeof(double));
 
 	for(i = 0; i < c_ord; i++) w2[i+1] = w[i];
@@ -800,11 +817,12 @@ mfunplwghts0_ret mfunwghtsi(int param_length, double* param, double siq){
 }
 
 void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3, 
-	       int* mask, int* siind, int* ngrad, int* ngrad0, int* maxcomp,
+	       int* mask, int* siind, int* ngrad0, int* maxcomp,
 	       int* maxit, 
 	       double* pen, double* grad_in, double* reltol,
 	       double* th, double* penIC, double* sigma2, double* vert, 
-	       double* orient, double* siq_in,
+	     //  double* orient, 
+	       double* siq_in,
 	       double* sigma2_ret, double* orient_ret, 
 	       int* order_ret, double* lev_ret, double* mix_ret){
 	
@@ -829,7 +847,8 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 	double ttt = 0, abstol = R_NegInf, intol = 1;
 	double alpha = 1, beta = 0.5, gamma = 2; //Optim parameters for Nelder Mead
 	double angles[2], dir[3];
-	double *param = 0, *param_red = 0, *param_ret = 0, *param_work = 0, *param_last = 0;
+	double *param = 0, *param_red = 0, *param_ret = 0, *param_work = 0, *param_last = 0,
+	*param_tmp = 0, *siq = 0, *w = 0;
 	
 	int fail;              // failure code for optim: zero is OK
 	double Fmin = 0.;          // minimal value of obj fct in optim 
@@ -839,7 +858,8 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 	siq_init = siq_in; penc = pen; grad = grad_in, ngrad0c = *ngrad0;
 	
 	// not yet initialized
-	double* w = Calloc(maxcompc, double);
+	//double* w = R_alloc(maxcompc, sizeof(double));
+	w = (double*) R_alloc(maxcompc, sizeof(double));
 	
 	//Gradient vectors corresponding to minima in spherical coordinates
 	
@@ -856,8 +876,14 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 		}
 	}
 	
-	param =  (double *) Calloc(param_length_init, double);
-	param_ret = (double *) Calloc(param_length_init, double);
+	param = (double *) R_alloc(param_length_init, sizeof(double));
+	param_ret = (double *) R_alloc(param_length_init, sizeof(double));
+	param_work = (double *) R_alloc(param_length_init, sizeof(double));
+	param_last = (double *) R_alloc(param_length_init, sizeof(double));
+	param_red = (double *) R_alloc(param_length_init, sizeof(double));
+	param_tmp = (double *) R_alloc(param_length_init, sizeof(double));
+	
+	siq = (double *) R_alloc(ngrad0c, sizeof(double));
 	
 	//initialize param
 	for(i=0; i<param_length_init; i++){
@@ -902,9 +928,6 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 			
 // 			Rprintf("param from orient:\n");
 			
-			// working param array
-			param_work = Calloc(param_length, double);
-			param_last = Calloc(param_length, double);
 			for(l = 0; l < param_length; l++){
 				param_work[l] = param[l];
 				param_last[l] = param[l];
@@ -917,7 +940,6 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 			for(k = maxcompc; k > 0; k--){
 				if(k < ord){
 					if(k != maxcompc){
-						double* param_tmp = Calloc(param_length, double);
 						for (l = 0; l < param_length; l++){
 							param_tmp[l] = param_work[l];
 						}
@@ -934,20 +956,16 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 							param_length=3*k+2;
 						}
 						
-						param_work = Calloc(param_length,double);
-						param_last = Calloc(param_length,double);
 						for(l= 0; l < param_length; l++){
 							param_work[l] = param_tmp[l];
 							param_last[l] = param_tmp[l];
 						}
 						
-						Free(param_tmp);
 					}
 					
 					if(optmethodc == 4){
 						int param_red_length;
 						if(methodc == 1){
-							param_red =  (double *) Calloc(3*k+1, double);
 							param_red_length = 3*k+1;
 							for(l = 0; l < param_red_length; l++){
 								param_red[l] = param_work[l];
@@ -963,7 +981,6 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 								}
 							}
 						}else{
-							param_red =  (double *) Calloc(3*k+2, double);
 							param_red_length = 3*k+2;
 							for(l = 0; l < param_red_length; l++){
 								param_red[l] = param_work[l];
@@ -998,7 +1015,6 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 					ret_val.value = 0;
 					ret_val.w = NULL;
 					
-					double* siq = Calloc(ngrad0c, double);
 					for(l=0;l<ngrad0c;l++){
 						siq[l] = siq_init[get_ind4d_img(i1,i2,i3,l)];
 					}
@@ -1037,8 +1053,6 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 						//other methods seem less numerically stable in this situation
 					} 
 					
-					Free(siq);
-					
 					if(optmethodc != 4){
 						value = ret_val.value;
 					}
@@ -1096,12 +1110,6 @@ void mixture2( int* method, int* optmethod, int* n1, int* n2, int* n3,
 	      R_CheckUserInterrupt();
 	    } // end i2
 	} // end i1
-	
-	Free(w);
-	Free(param);
-	Free(param_ret);
-	Free(param_work);
-	Free(param_red);
 }
 
 void mixturetest(double* test2d, double *test3d, double *test4d, double* test5d,
